@@ -1,17 +1,10 @@
 import sys
-import sysconfig
 import os
 import logging
-import re
-import errno
-import shlex
-import subprocess
 import itertools
 import json
-import traceback
 import concurrent.futures
 
-from collections import namedtuple, deque
 from xml.sax.saxutils import escape
 from distutils import version
 
@@ -26,8 +19,7 @@ except ImportError:
 from PyQt5.QtWidgets import (
     QWidget, QDialog, QLabel, QLineEdit, QTreeView, QHeaderView,
     QTextBrowser, QDialogButtonBox, QProgressDialog,
-    QVBoxLayout, QStyle, QStyledItemDelegate, QStyleOptionViewItem,
-    QApplication, QHBoxLayout,  QPushButton, QFormLayout
+    QVBoxLayout, QHBoxLayout
 )
 
 from PyQt5.QtGui import (
@@ -36,15 +28,14 @@ from PyQt5.QtGui import (
 
 from PyQt5.QtCore import (
     QSortFilterProxyModel, QItemSelectionModel,
-    Qt, QObject, QMetaObject, QEvent, QSize, QTimer, QThread, Q_ARG,
-    QSettings)
+    Qt, QSize, QTimer, QThread)
 from PyQt5.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
 
 from urllib.request import urlopen
 
 from orangecanvas.gui.utils import message_warning, message_information, \
                         message_critical as message_error
-from orangecanvas.help.manager import get_dist_meta, trim, parse_meta
+from orangecanvas.help.manager import get_dist_meta, trim
 
 from orangecanvas.resources import package_dirname
 
@@ -96,7 +87,7 @@ from oasys.application.addons import get_meta_from_archive, cleanup, TristateChe
 from oasys.application.addons import Installed, Installable, Available
 
 def is_updatable(item):
-    if isinstance(item, Available) or item.installable is None:
+    if isinstance(item, Available) or isinstance(item, Installable) or item.installable is None:
         return False
     inst, dist = item
     try:
@@ -108,6 +99,10 @@ def is_updatable(item):
             if MAX_VERSION[dist.project_name] is None: return True
             else: return version.LooseVersion(MAX_VERSION[dist.project_name]) >= version.LooseVersion(inst.version)
 
+def is_installable(item):
+    if isinstance(item, Available) or isinstance(item, Installable) or item.installable is None: return True
+    else: return False
+
 class InternalLibrariesManagerWidget(QWidget):
 
     statechanged = Signal()
@@ -198,12 +193,14 @@ class InternalLibrariesManagerWidget(QWidget):
                            (Qt.ItemIsTristate if updatable else 0))
             item1.setEnabled(False)
 
-            if installed and updatable:
-                item1.setCheckState(Qt.Checked)
-            elif installed:
-                item1.setCheckState(Qt.Checked)
-            else:
-                item1.setCheckState(Qt.Unchecked)
+            item1.setCheckState(Qt.Checked)
+
+            #if installed and updatable:
+            #    item1.setCheckState(Qt.Checked)
+            #elif installed:
+            #    item1.setCheckState(Qt.Checked)
+            #else:
+            #    item1.setCheckState(Qt.Unchecked)
 
             item2 = QStandardItem(cleanup(name))
 
@@ -222,6 +219,8 @@ class InternalLibrariesManagerWidget(QWidget):
 
             if updatable:
                 item4.setText("Update")
+            elif not installed:
+                item4.setText("Install")
 
             model.appendRow([item1, item2, item3, item4])
 
@@ -649,4 +648,12 @@ def list_available_versions():
 def list_installed_internal_libraries():
     workingset = pkg_resources.WorkingSet(sys.path)
 
-    return [workingset.by_key[internal_library.lower()] for internal_library in INTERNAL_LIBRARIES]
+    installed_internal_libraries = []
+
+    for internal_library in INTERNAL_LIBRARIES:
+        try:
+            installed_internal_libraries.append(workingset.by_key[internal_library.lower()])
+        except KeyError:
+            pass
+
+    return installed_internal_libraries
